package gregapi.block;

import static gregapi.data.CS.ALL_SIDES_VALID;
import static gregapi.data.CS.SIDE_TOP;
import static gregapi.data.CS.T;
import static gregapi.data.CS.TOOL_LOCALISER_PREFIX;
import gregapi.data.LH;
import gregapi.data.MD;
import gregapi.util.ST;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.IIcon;
import net.minecraft.world.World;

public class ItemBlockBase extends ItemBlock {
	public final BlockBase mPlaceable;
	
	public ItemBlockBase(Block aBlock) {
		super(aBlock);
		mPlaceable = (BlockBase)aBlock;
		setMaxDamage(0);
		setHasSubtypes(T);
		setCreativeTab(CreativeTabs.tabBlock);
	}
	
	@Override
	public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {
		super.addInformation(aStack, aPlayer, aList, aF3_H);
		short aMeta = ST.meta(aStack);
		mPlaceable.addInformation(aStack, aMeta, aPlayer, aList, aF3_H);
		if (mPlaceable.doesWalkSpeed(aMeta)) aList.add(LH.Chat.CYAN + LH.get(LH.TOOLTIP_WALKSPEED));
		if (!mPlaceable.canCreatureSpawn(aMeta) && mPlaceable.getCollisionBoundingBoxFromPool(aPlayer.worldObj, 0, 0, 0) != null) aList.add(LH.Chat.CYAN + LH.get(LH.TOOLTIP_SPAWNPROOF));
		
		if (MD.GC.mLoaded) {
			byte tCount = 0;
			for (byte tSide : ALL_SIDES_VALID) if (mPlaceable.isSealable(aMeta, tSide)) tCount++;
			if (tCount >= 6) {
				aList.add(LH.Chat.GREEN  + LH.get(LH.TOOLTIP_SEALABLE_ANY));
			} else if (mPlaceable.isOpaqueCube()) {
				aList.add(LH.Chat.ORANGE + LH.get(LH.TOOLTIP_SEALABLE_BUGGED));
			} else if (tCount > 0) {
				aList.add(LH.Chat.YELLOW + LH.get(LH.TOOLTIP_SEALABLE_SOME));
			}
		}
		
		if (mPlaceable.useGravity(aMeta)) aList.add(LH.Chat.ORANGE + LH.get(LH.TOOLTIP_GRAVITY));
		aList.add(LH.Chat.WHITE + LH.get(LH.TOOLTIP_BLASTRESISTANCE) + LH.Chat.ORANGE + mPlaceable.getExplosionResistance(aMeta));
		aList.add(LH.Chat.DGRAY + LH.get(LH.TOOL_TO_HARVEST) + ": " + LH.Chat.WHITE + LH.get(TOOL_LOCALISER_PREFIX + mPlaceable.getHarvestTool(aMeta), "Unknown") + " (" + mPlaceable.getHarvestLevel(aMeta) + ")");
	}
	
	@Override public IIcon getIconFromDamage(int aMeta) {return mPlaceable.getIcon(SIDE_TOP, aMeta);}
	@Override public String getUnlocalizedName(ItemStack aStack) {return mPlaceable.mNameInternal + "." + getDamage(aStack);}
	@Override public boolean placeBlockAt(ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, int aSide, float aHitX, float aHitY, float aHitZ, int aMetaData) {return aWorld.setBlock(aX, aY, aZ, mPlaceable, aMetaData, 3);}
	@Override public int getItemStackLimit(ItemStack aStack) {return mPlaceable.getItemStackLimit(aStack);}
	@Override public int getMetadata(int aMeta) {return aMeta;}
	@Override public ItemStack onItemRightClick(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {return mPlaceable.onItemRightClick(aStack, aWorld, aPlayer);}
}