package gregapi.worldgen;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.MultiTileEntityRegistry;
import gregapi.data.CS.BlocksGT;
import gregapi.data.CS.ConfigsGT;
import gregapi.data.OP;
import gregapi.oredict.OreDictManager;
import gregapi.oredict.OreDictMaterial;
import gregapi.util.ST;
import gregapi.util.UT;
import gregapi.util.WD;

import java.util.List;
import java.util.Random;
import java.util.Set;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraft.world.chunk.Chunk;

/**
 * @author Gregorius Techneticies
 */
public class WorldgenOresBedrock extends WorldgenObject {
	public final int mProbability;
	public final OreDictMaterial mMaterial;
	
	public static boolean CAN_GENERATE_BEDROCK_ORE = T;
	
	public WorldgenOresBedrock(String aName, boolean aDefault, int aProbability, OreDictMaterial aPrimary, List... aLists) {
		super(aName, aDefault, aLists);
		mProbability		= Math.max(1,			ConfigsGT.WORLDGEN.get(mCategory, "Probability"		, aProbability));
		mMaterial			= OreDictMaterial.get(	ConfigsGT.WORLDGEN.get(mCategory, "Ore"				, aPrimary.mNameInternal));
		
		if (mEnabled) OreDictManager.INSTANCE.triggerVisibility("ore"+mMaterial.mNameInternal);
		
		if (mMaterial.mID <= 0) {
			ERR.println("The Material is not valid for Ores: " + mMaterial);
			mInvalid = T;
		}
	}
	
	@Override
	public void reset(World aWorld, Chunk aChunk, int aDimType, int aMinX, int aMinZ, int aMaxX, int aMaxZ, Random aRandom, BiomeGenBase[][] aBiomes, Set<String> aBiomeNames) {
		CAN_GENERATE_BEDROCK_ORE = T;
	}
	
	@Override
	public boolean generate(World aWorld, Chunk aChunk, int aDimType, int aMinX, int aMinZ, int aMaxX, int aMaxZ, Random aRandom, BiomeGenBase[][] aBiomes, Set<String> aBiomeNames) {
		if (!CAN_GENERATE_BEDROCK_ORE || aRandom.nextInt(mProbability) != 0) return F;
		if (aWorld.getBlock(aMinX+8, 0, aMinZ+8) == Blocks.bedrock) {
			CAN_GENERATE_BEDROCK_ORE = F;
			
			
			ItemStack tRock = OP.rockGt.mat(mMaterial, 1);
			if (ST.valid(tRock)) {
				MultiTileEntityRegistry tRegistry = MultiTileEntityRegistry.getRegistry("gt.multitileentity");
				if (tRegistry != null) {
					for (int i = 0; i < 16; i++) {
						int tX = aMinX+aRandom.nextInt(32)-8, tZ = aMinZ+aRandom.nextInt(32)-8;
						for (int tY = 90; tY > 62; tY--) {
							Block tContact = aWorld.getBlock(tX, tY, tZ);
							if (tContact.getMaterial().isLiquid()) break;
							if (!tContact.isOpaqueCube()) continue;
							if (tContact.getMaterial() != Material.grass && tContact.getMaterial() != Material.ground && tContact.getMaterial() != Material.sand && tContact.getMaterial() != Material.rock) break;
							if (WD.easyRep(aWorld, tX, tY+1, tZ)) tRegistry.mBlock.placeBlock(aWorld, tX, tY+1, tZ, SIDE_UNKNOWN, (short)32757, aRandom.nextInt(3)!=0?ST.save(UT.NBT.make(), NBT_VALUE, tRock):UT.NBT.make(), F, T);
							break;
						}
					}
				}
			}
			
			
			int[] tDistances = new int[] {0, 3, 6, 8, 9, 7, 4};
			for (int tY = 1; tY < 7; tY++) for (int tX = -tDistances[tY]; tX <= tDistances[tY]; tX++) for (int tZ = -tDistances[tY]; tZ <= tDistances[tY]; tZ++) {
				WD.removeBedrock(aWorld, aMinX+8+tX, tY, aMinZ+8+tZ);
				switch(aRandom.nextInt(6)) {
				case 0: 		WD.setOre(aWorld, aMinX+8+tX, tY, aMinZ+8+tZ, mMaterial); break;
				case 1: case 2:	WD.setSmallOre(aWorld, aMinX+8+tX, tY, aMinZ+8+tZ, mMaterial); break;
				}
			}
			for (int tX = -2; tX <= 2; tX++) for (int tZ = -2; tZ <= 2; tZ++) {
				switch(aRandom.nextInt(6)) {
				case 0: 		BlocksGT.oreBedrock			.placeBlock(aWorld, aMinX+8+tX, 0, aMinZ+8+tZ, (byte)6, mMaterial.mID, null, F, T); break;
				case 1: case 2:	BlocksGT.oreSmallBedrock	.placeBlock(aWorld, aMinX+8+tX, 0, aMinZ+8+tZ, (byte)6, mMaterial.mID, null, F, T); break;
				}
			}
			// At least one Ore Block must be there. So place a large one in the Center.
			BlocksGT.oreBedrock.placeBlock(aWorld, aMinX+8, 0, aMinZ+8, (byte)6, mMaterial.mID, null, F, T);
		}
		return T;
	}
}